# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
import json
import os
import sys
from test.test_support import EnvironmentVarGuard

from unittest import TestCase
from rauth import OAuth1Session
import rauth
import requests
from requests.structures import CaseInsensitiveDict
from quickbook3 import QuickBooks

try:
    from unittest import mock
except ImportError:
    import mock

from io import BytesIO


class BaseCase(TestCase):

    COMPANY_ID = 'company_id'
    CONSUMER_KEY = 'consumer_key'
    CONSUMER_SECRET = 'consumer_secret'
    ACCESS_TOKEN = 'access_token'
    ACCESS_SECRET = 'access_secret'

    CREDENTIAL_FILE = 'tests/fixtures/credfile'
    INCOMPLETE_CREDENTIAL_FILE = 'tests/fixtures/incomplete_credfile'

    PRODUCTION_URL = "https://quickbooks.api.intuit.com/v3"

    SANDBOX_URL = "https://sandbox-quickbooks.api.intuit.com/v3"

    def setUp(self):
        self.env = EnvironmentVarGuard()
        self.args = ()
        self.conf = {
            'header_auth': True,
            'headers': {
                'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
        }
        self.mock = mock.patch.object(rauth.OAuth1Session, 'request')
        self.request = self.mock.start()
        self.qbclient = None

    def tearDown(self):
        self.mock.stop()

    def set_default_client(self, qbclient=None):
        self.qbclient = qbclient or QuickBooks(company_id=self.COMPANY_ID,
                                               cred_file=self.CREDENTIAL_FILE)

        self.conf['realm'] = self.qbclient.company_id

    def response(self, resource, body=None, status_code=200, encoding='utf-8',
                 is_array=False, **headers):

        r = requests.Response()
        r.status_code = status_code
        r.encoding = encoding

        if body:
            r.json = lambda: body
        else:
            r.json = lambda: {resource: {}}

        r.headers = CaseInsensitiveDict({'Content-Type': 'application/json'})
        r.headers.update(headers)

        self.request.return_value = r

    def request_assertions(self, response):
        assert self.request.called is True
        assert isinstance(response, dict)
        conf = self.conf.copy()

        args, kwargs = self.request.call_args

        assert self.args == args

        for k in self.conf:
            assert k in kwargs
            assert self.conf[k] == kwargs[k]

        self.request.reset_mock()
        self.conf = conf

    def _get_url(self, resource, resource_id=None):
        if resource_id:
            return os.path.join(self.PRODUCTION_URL, 'company',
                                self.qbclient.company_id, resource, resource_id)
        else:
            return os.path.join(self.PRODUCTION_URL, 'company',
                                self.qbclient.company_id, resource)

    def delete(self, resource, resource_id=None):
        self.args = ('DELETE', self._get_url(resource, resource_id))

    def get(self, resource, resource_id=None):
        self.args = ('GET', self._get_url(resource, resource_id))

    def patch(self, resource, resource_id=None):
        self.args = ('PATCH', self._get_url(resource, resource_id))

    def post(self, resource, resource_id=None):
        self.args = ('POST', self._get_url(resource, resource_id))

    def put(self, resource, resource_id=None):
        self.args = ('PUT', self._get_url(resource, resource_id))


class RequestsBytesIO(BytesIO):
    def read(self, chunk_size, *args, **kwargs):
        return super(RequestsBytesIO, self).read(chunk_size)



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from unittest import TestCase
import requests
from quickbook3 import *

try:
    from unittest import mock
except ImportError:
    import mock


ERROR_MSG_MAP = {
    AuthenticationError: 'User authentication Failed',
    PermissionDenied: 'permission',
    NotFoundError: 'Not Found',
    ServiceUnavailable: 'Service Unavailable',
    ServerError: 'Server Error'
}

class TestResponseParser(TestCase):

    def create_response_parser(self, status_code=200, response_body=None,
                               content_type='application/json', **kwargs):

        resp = requests.Response()
        resp.status_code = status_code
        response_body = response_body or {}
        resp.json = lambda: response_body
        for key, val in kwargs.items():
            setattr(resp, key, val)

        if content_type:
            resp.headers['content-type'] = content_type

        return ResponseParser(resp)

    def test_parse_response_success(self):
        body = {'response': 'success'}
        parser = self.create_response_parser(200, body)
        self.assertEqual(body, parser.parse())

    def test_is_xml_response_success(self):
        parser = self.create_response_parser(content_type='application/xml')
        assert parser.is_xml_response()

    def test_is_xml_response_failure(self):
        parser = self.create_response_parser()
        assert not parser.is_xml_response()

    def _test_http_exception(self, status_code, exception_type,
                             reason=None,
                             assert_status=False):

        reason = reason or ERROR_MSG_MAP[exception_type]
        parser = self.create_response_parser(status_code=status_code, reason=reason)
        with self.assertRaises(exception_type) as cm:
            parser.parse()

        err = str(cm.exception)

        if assert_status:
            assert str(status_code) in err

        assert reason.lower() in err.lower()

    def test_parse_response_unknown_error(self):
        self._test_http_exception(409, UnknownError, 'Conflict')

    def test_parse_response_authentication_error(self):
        self._test_http_exception(401, AuthenticationError)

    def test_parse_response_authorization_error(self):
        self._test_http_exception(403, PermissionDenied)

    def test_parse_response_notfound_error(self):
        self._test_http_exception(404, NotFoundError)

    def test_parse_response_server_error(self):
        self._test_http_exception(500, ServerError)

    def test_parse_response_service_unavailable(self):
        self._test_http_exception(503, ServiceUnavailable)

    def _test_bad_request_error(self, fault_type, exception_type, reason=None):
        reason = reason or ERROR_MSG_MAP[exception_type]
        parser = self.create_response_parser(status_code=400,
                                             response_body={
                                                 'Fault': {
                                                     'type': fault_type,
                                                     'Error': reason
                                                 }
                                             })

        with self.assertRaises(exception_type) as cm:
            parser.parse()

        err = str(cm.exception)
        assert reason.lower() in err.lower()

    def test_bad_request_authentication_error(self):
        self._test_bad_request_error('authentication', AuthenticationError)

    def test_bad_request_authorization_error(self):
        self._test_bad_request_error('authorization', PermissionDenied)

    def test_bad_request_service_fault(self):
        self._test_bad_request_error('SERVICEFAULT', ServerError)

    def test_bad_request_system_fault(self):
        self._test_bad_request_error('SYSTEMFAULT', ServerError)

    def test_bad_request_validation_error(self):
        errors = [
            {
                'Detail': 'First Error'
            },
            {
                'Detail': 'Second Error'
            }
        ]
        parser = self.create_response_parser(status_code=400,
                                             response_body={
                                                 'Fault': {
                                                     'type': 'validationfault',
                                                     'Error': errors
                                                 }
                                             })

        with self.assertRaises(ValidationFault) as cm:
            parser.parse()

        err_msg = str(cm.exception)

        for err in errors:
            assert err['Detail'] in err_msg




# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from quickbook3 import QueryBuilder, InvalidQueryError
from tests.utils import BaseCase


class TestQueryBuilder(BaseCase):

    def test_get_entity(self):
        qb = QueryBuilder('company')
        entity = qb.get_entity()
        self.assertEqual(entity, 'company')

    def test_select_chainable(self):
        qb = QueryBuilder('company')
        ret = qb.select("a, b")
        self.assertEqual(qb, ret)

    def test_select_columns_list(self):
        qb = QueryBuilder('company')
        columns = ['a', 'b']
        columns_str = ", ".join(columns)
        qb.select(columns)
        self.assertEqual(qb.columns, columns_str)

    def test_select_columns_tuple(self):
        qb = QueryBuilder('company')
        columns = ('a', 'b')
        columns_str = ", ".join(columns)
        qb.select(columns)
        self.assertEqual(qb.columns, columns_str)

    def test_select_columns_str(self):
        qb = QueryBuilder('company')
        columns = 'a,b'
        qb.select(columns)
        self.assertEqual(qb.columns, columns)

    def test_get_columns_all(self):
        qb = QueryBuilder('company')
        self.assertEqual(qb.get_columns(), '*')

    def test_get_columns_specified(self):
        qb = QueryBuilder('company')
        columns = 'a, b'
        qb.select(columns)
        self.assertEqual(qb.get_columns(), columns)

    def test_where_chainable(self):
        self._test_clause_chainable("where", "a", where=False)

    def test_where_clause(self):
        qb = QueryBuilder('company')
        qb.where("a")
        self.assertEqual(qb.incomplete_filter_flag, True)
        self.assertEqual(len(qb.filters), 1)
        self.assertEqual(qb.filters[0], 'a')

    def test_incomplete_where_clause_raises_error(self):
        qb = QueryBuilder('company')
        qb.where("a")
        self.assertRaisesRegexp(InvalidQueryError,
                                r'Incomplete Filter Clause', qb.build)

    def test_equal_clause_raise_error_without_where(self):
        qb = QueryBuilder('company')
        self.assertRaisesRegexp(InvalidQueryError,
                                r'preceeded by where',
                                qb.equals, 'a')

    def test_equal_clause_chainable(self):
        self._test_clause_chainable("equals", "5")

    def test_equal_clause_bool(self):
        qb = QueryBuilder('company')
        qb.where("a").equals(True)
        self._test_clause(qb, "a", '=', "true")

    def test_equal_clause_none(self):
        qb = QueryBuilder('company')
        qb.where("a").equals(None)
        self._test_clause(qb, "a", '=', "''")

    def test_equal_clause(self):
        qb = QueryBuilder('company')
        qb.where("a").equals("b")
        self._test_clause(qb, "a", '=', "'b'")

    def test_contains_clause_list(self):
        qb = QueryBuilder('company')
        values = [1, 2, 3]
        qb.where('a').contains(values)
        self._test_clause(qb, "a", 'in', str(values))

    def test_contains_clause_tuple(self):
        qb = QueryBuilder('company')
        values = (1, 2, 3)
        qb.where('a').contains(values)
        self._test_clause(qb, "a", 'in', str(values))

    def test_contains_clause_raises_exception_str_value(self):
        qb = QueryBuilder('company')
        values = "fgf"
        qb = qb.where('a')
        self.assertRaisesRegexp(InvalidQueryError, "list/tuple of values",
                                qb.contains, values)

    def test_gt_clause(self):
        qb = QueryBuilder('company')
        qb.where("a").gt("5")
        self._test_clause(qb, "a", ">", "'5'")
        
    def test_gt_chainable(self):
        return self._test_clause_chainable("gt", "5")

    def test_gte_clause(self):
        qb = QueryBuilder('company')
        qb.where("a").gte("5")
        self._test_clause(qb, "a", ">=", "'5'")
        
    def test_gte_chainable(self):
        return self._test_clause_chainable("gte", "5")
        
    def test_lt_clause(self):
        qb = QueryBuilder('company')
        qb.where("a").lt("5")
        self._test_clause(qb, "a", "<", "'5'")
        
    def test_lt_chainable(self):
        return self._test_clause_chainable("lt", "5")        

    def test_lte_clause(self):
        qb = QueryBuilder('company')
        qb.where("a").lte("5")
        self._test_clause(qb, "a", "<=", "'5'")
        
    def test_lte_chainable(self):
        return self._test_clause_chainable("lte", "5")

    def test_like_clause(self):
        qb = QueryBuilder('company')
        qb.where("a").like("5")
        self._test_clause(qb, "a", "Like", "'5'")

    def test_like_clause_raise_error_on_non_str(self):
        qb = QueryBuilder('company')
        qb.where("a")
        self.assertRaisesRegexp(InvalidQueryError, r"value be of type string",
                                qb.like, True)

    def test_like_chainable(self):
        return self._test_clause_chainable("like", "5")

    def _test_clause_chainable(self, clause, param, where=True):
        qb = QueryBuilder('company')
        if where:
            qb.where("a")
        ret = getattr(qb, clause)(param)
        self.assertEqual(qb, ret)

    def _test_clause(self, qb, lhs, op, rhs):
        self.assertEqual(qb.incomplete_filter_flag, False)
        self.assertEqual(qb.filters[0], "{} {} {}".format(lhs, op, rhs))





# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
import logging
from quickbook3 import QuickBooks, MissingCredentialsException, \
    InvalidResourceError
from tests.utils import BaseCase


class TestQuickbooks(BaseCase):

    def test_init(self):
        self.set_default_client(QuickBooks(company_id=self.COMPANY_ID,
                                           consumer_key=self.CONSUMER_KEY,
                                           consumer_secret=self.CONSUMER_SECRET,
                                           access_token=self.ACCESS_TOKEN,
                                           access_token_secret=self.ACCESS_SECRET))

        self.assertEqual(self.qbclient.company_id, self.COMPANY_ID)
        self._assert_credentials()
        self.assertEqual(self.qbclient.sandbox_mode, False)
        self.assertIsNone(self.qbclient.logger)
        self.assertEqual(self.qbclient.log_level, logging.ERROR)

    def test_missing_consumer_key_exception(self):
        self.assertRaises(MissingCredentialsException,
                          QuickBooks,
                          company_id=self.COMPANY_ID,
                          consumer_secret=self.CONSUMER_SECRET,
                          access_token=self.ACCESS_TOKEN,
                          access_token_secret=self.ACCESS_SECRET)

    def test_missing_access_token_secret_exception(self):
        self.assertRaises(MissingCredentialsException,
                          QuickBooks,
                          company_id=self.COMPANY_ID,
                          consumer_key=self.CONSUMER_KEY,
                          consumer_secret=self.CONSUMER_SECRET,
                          access_token=self.ACCESS_TOKEN)

    def test_read_creds_from_file(self):
        self.set_default_client(QuickBooks(company_id=self.COMPANY_ID,
                                           cred_file=self.CREDENTIAL_FILE))
        self._assert_credentials()

    def test_exception_read_incomplete_creds(self):
        self.assertRaises(MissingCredentialsException,
                          QuickBooks,
                          company_id=self.COMPANY_ID,
                          cred_file=self.INCOMPLETE_CREDENTIAL_FILE)

    def test_credentials_from_environment(self):
        self.env.set('QB_CONSUMER_KEY', self.CONSUMER_KEY)
        self.env.set('QB_CONSUMER_SECRET', self.CONSUMER_SECRET)
        self.env.set('QB_ACCESS_TOKEN', self.ACCESS_TOKEN)
        self.env.set('QB_ACCESS_TOKEN_SECRET', self.ACCESS_SECRET)

        with self.env:
            self.set_default_client(QuickBooks(company_id=self.COMPANY_ID))
            self._assert_credentials()

    def _assert_credentials(self):
        self.assertEqual(self.qbclient.consumer_key, self.CONSUMER_KEY)
        self.assertEqual(self.qbclient.consumer_secret, self.CONSUMER_SECRET)
        self.assertEqual(self.qbclient.access_token, self.ACCESS_TOKEN)
        self.assertEqual(self.qbclient.access_token_secret, self.ACCESS_SECRET)

    def test_base_url_sandbox(self):
        self.set_default_client(QuickBooks(company_id=self.COMPANY_ID,
                                           sandbox_mode=True,
                                           cred_file=self.CREDENTIAL_FILE))

        self.assertEqual(self.qbclient.base_url_v3, self.SANDBOX_URL)

    def test_base_url_production(self):
        self.set_default_client()
        self.assertEqual(self.qbclient.base_url_v3, self.PRODUCTION_URL)

    def test_crud_url_list(self):
        self.set_default_client()
        customer_url = self.PRODUCTION_URL + '/company/' + self.COMPANY_ID + '/customer'
        self.assertEqual(self.qbclient._get_crud_url('customer'), customer_url)

    def test_crud_url_detail(self):
        self.set_default_client()
        customer_id = 'customer_id'
        customer_url = self.PRODUCTION_URL + '/company/' + self.COMPANY_ID + '/customer/' + customer_id
        self.assertEqual(self.qbclient._get_crud_url('customer', customer_id), customer_url)

    def test_invalid_resource(self):
        self.set_default_client()
        self.assertRaises(InvalidResourceError, self.qbclient._get_crud_url, 'invalid')

    def test_create_resource(self):
        resource_dict = {'customer': {'name': 'Name'}}
        self.set_default_client()
        self.response('customer')
        self.post('customer')
        self.conf['json'] = resource_dict
        resp = self.qbclient.create('customer', resource_dict)
        self.request_assertions(resp)

    def test_read_resource(self):
        resource_id = 'customer_id'
        self.set_default_client()
        self.response('customer')
        self.get('customer', resource_id)

        resp = self.qbclient.read('customer', resource_id)
        self.request_assertions(resp)

    def test_update_resource(self):
        resource_dict = {'customer': {'name': 'Name'}}
        self.set_default_client()
        self.response('customer')
        self.post('customer')
        self.conf['json'] = resource_dict
        resp = self.qbclient.update('customer', resource_dict)
        self.request_assertions(resp)

    def test_delete_resource(self):
        resource_dict = {'customer': {'name': 'Name'}}
        self.set_default_client()
        self.response('customer')
        self.post('customer')
        self.conf['json'] = resource_dict
        self.conf['params'] = {'operation': 'delete'}
        resp = self.qbclient.delete('customer', resource_dict)
        self.request_assertions(resp)


